unit FFT2Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtDlgs, Vcl.StdCtrls, Vcl.ExtCtrls, system.Math;

type
  D2array = array of array of Double;
  B2array = array of array of Byte;
  TDarray = array[0..0] of Double;
  PDarray = ^TDarray;

  TForm1 = class(TForm)
    Image1: TImage;
    FileOpenBtn: TButton;
    OpenPictureDialog1: TOpenPictureDialog;
    FFT2Btn: TButton;
    FilterBtn: TButton;
    LabeledEdit1: TLabeledEdit;
    LabeledEdit2: TLabeledEdit;
    CheckBox1: TCheckBox;
    ColorRadio: TRadioGroup;
    procedure FileOpenBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FFT2BtnClick(Sender: TObject);
    procedure FilterBtnClick(Sender: TObject);
  private
    { Private 錾 }
    function FFT2(var a_rl, a_im: D2array; inv, Y_EXP, X_EXP: Integer): Integer;
    procedure cstb(length: Integer; inv: integer; var sin_tbl, cos_tbl: array of double);
    procedure fft1core(a_rl, a_im: PDarray; length, ex: integer;
                    sin_tbl, cos_tbl, buf: PDarray);
    procedure birv(a: PDarray; length, ex: Integer; b: PDarray);
    procedure rvmtx1(var a, b: D2array; xsize, ysize: Integer);
    procedure rvmtx2(var a, b: D2array; xsize, ysize: integer);
    function fftimage(image_in, image_out: B2array): Integer;
    function fftfilter(image_in, image_out: B2array; a, b: Integer): Integer;
  public
    { Public 錾 }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const
  OpenFileFilter =
    '摜t@C|*.png;*.jpg;*.gif;*.bmp;*.tif;*.ico;*.wdp'+
    '|*.png|*.png' +
    '|*.jpg|*.jpg' +
    '|*.gif|*.gif' +
    '|*.bmp|*.bmp' +
    '|*.tif|*.tif' +
    '|*.ico|*.ico' +
    '|*.wdp|*.wdp';

  SaveFileFilter =
    '摜t@C|*.png;*.jpg;*.gif;*.bmp;*.tif;*.wdp' +
    '|*.png|*.png' +
    '|*.jpg|*.jpg' +
    '|*.gif|*.gif' +
    '|*.bmp|*.bmp' +
    '|*.tif|*.tif' +
    '|*.wdp|*.wdp';

  ImageHW   = 384;    // \gTCY

var
  InputDBitmap  : TBitmap;                    // ̓f[^[\prbg}bv
  OutputBitmap  : TBitmap;                    // ]摜\prbg}bv
  GHeight       : Integer;                    // ̓f[^[摜
  GWidth        : Integer;                    // ̓f[^[摜

  VRect         : Trect;                      // \TCYݒp
  InFilename    : string;                     // ̓t@C
  InpBMat       : B2array;
  InpGMat       : B2array;
  InpRMat       : B2array;
  OutBMat       : B2array;
  OutGMat       : B2array;
  OutRMat       : B2array;
  OPT           : integer;          // OPT = 1@wIcesij
                                    // OPT = 0@ʏ̂cesi[j


const
  kr  = 0.299;                     // R PxϊW
  kg  = 0.587;                     // G PxϊW
  kb  = 0.114;                     // B PxϊW

//--- rvmtx2 --- Qf[^̓]u -------------------------------------------
//    a:        Q̓f[^
//    b:        Qo̓f[^
//    xsize:    f[^
//    ysize:    f[^
//-----------------------------------------------------------------------------
procedure TForm1.rvmtx2(var a, b: D2array; xsize, ysize: integer);
var
  i, j: integer;
begin
  for i := 0 to ysize - 1 do
    for j := 0 to xsize - 1 do b[i, j] := a[j, i];
end;

//--- rvmtx1 --- Qf[^̓]u -------------------------------------------
//    a:        Q̓f[^
//    b:        Qo̓f[^
//    xsize:    f[^
//    ysize:    f[^
//-----------------------------------------------------------------------------
procedure TForm1.rvmtx1(var a, b: D2array; xsize, ysize: Integer);
var
  i, j: integer;
begin
  for i := 0 to ysize - 1 do
    for j := 0 to  xsize - 1 do b[j, i] := a[i, j];
end;

//--- birv --- f[^̕׊ -----------------------------------------------
//    a:       f[^̔z
//    length:  f[^
//    ex:      f[^Q̗ݏŗ^܂(2exj
//    b:       Ɨpobt@
//-----------------------------------------------------------------------------
procedure TForm1.birv(a: PDarray; length, ex: Integer; b: PDarray);
var
  i, ii, k, bit : Integer;
begin
  for i := 0 to length - 1 do begin
    ii := i;
    bit:= 0;
    for k := 0 to ex do begin
      bit := (ii and 1) or bit;
      if k = ex - 1 then break;
      bit := bit shl 1;
      ii := ii shr 1;
    end;
    b[i] := a[bit];
  end;
  for i := 0 to length - 1 do a[i] := b[i];
end;

//--- fft1core --- PeešvZ̊jɂȂ镔 ---------------------------
//    a_rl:    f[^io͌pj
//    a_im:    f[^io͌pj
//    ex:        f[^Q̗ݏŗ^܂(2exj
//    sin_tbl:    SINvZpe[u
//    cos_tbl:    COSvZpe[u
//-----------------------------------------------------------------------------
procedure TForm1.fft1core(a_rl, a_im: PDarray; length, ex: integer;
        sin_tbl, cos_tbl, buf: PDarray);
var
  i, j, k, w, j1, j2    : Integer;
  numb, lenb, timb      : Integer;
  xr, xi, yr, yi, nrml  : Double;
begin
  if OPT = 1 then begin
    j := 1;
    Repeat                          // for j = 1 to length - 1 step 2 do Ɠ
      a_rl[j] := - a_rl[j];
      a_im[j] := - a_im[j];
      j := j + 2;
    until j > length - 1;
  end;
  numb := 1;
  lenb := length;
  for i := 0 to ex - 1 do begin
    lenb := lenb div 2;
    timb := 0;
    for j := 0 to numb - 1 do begin
      w := 0;
      for k := 0 to lenb - 1 do  begin
        j1 := timb + k;
        j2 := j1 + lenb;
        xr := a_rl[j1];
        xi := a_im[j1];
        yr := a_rl[j2];
        yi := a_im[j2];
        a_rl[j1] := xr + yr;
        a_im[j1] := xi + yi;
        xr := xr - yr;
        xi := xi - yi;
        a_rl[j2] := xr * cos_tbl[w] - xi * sin_tbl[w];
        a_im[j2] := xr * sin_tbl[w] + xi * cos_tbl[w];
        w := w + numb;
      end;
      timb := timb + 2 * lenb;
    end;
    numb := numb * 2;
  end;
  birv(a_rl, length, ex, buf);      // f[^̕׊
  birv(a_im, length, ex, buf);      // f[^̕׊
  if OPT = 1 then begin
    j := 1;
    Repeat                          // for j = 1 to length - 1 step 2 do Ɠ
      a_rl[j] := - a_rl[j];
      a_im[j] := - a_im[j];
      j := j + 2;
    until j > length - 1;
  end;
  nrml := 1.0 / sqrt(length);
  for i := 0 to length - 1 do begin
    a_rl[i] := a_rl[i] * nrml;
    a_im[i] := a_im[i] * nrml;
  end;
end;

//--- cstb --- SIN,COSe[u쐬 --------------------------------------------
//    length:        f[^
//    inv:        1: ces, -1: tces
//    sin_tbl:    SINvZpe[u
//    cos_tbl:    COSvZpe[u
//-----------------------------------------------------------------------------
procedure TForm1.cstb(length: Integer; inv: Integer; var sin_tbl, cos_tbl: array of double);
var
  i : Integer;
  xx, arg : Double;
begin
  xx := - PI * 2.0 / length;
  if inv < 0 then xx := - xx;
  for i := 0 to length - 1 do begin
    arg := i * xx;
    sin_tbl[i] := sin(arg);
    cos_tbl[i] := cos(arg);
  end;
end;

//--- fft2 --- Qees̎s ---------------------------------------------
//   iGWidthCGHeightQׂ̂̏ꍇɌ܂j
//    a_rl:    f[^io͌pj
//    a_im:    f[^io͌pj
//    inv:    1: cesC-1: tces
//    X_EXP:  ex:        f[^Q̗ݏŗ^܂(2exj
//    Y_EXP:  ey:        f[^Q̗ݏŗ^܂(2eyj
//-----------------------------------------------------------------------------
function TForm1.FFT2(var a_rl, a_im: D2array; inv, Y_EXP, X_EXP: Integer): Integer;
var
  b_rl      : D2array;              // f[^]uƗpzij
  b_im      : D2array;              // f[^]uƗpzi)
  hsin_tbl  : array of Double;      // pSINvZpe[u
  hcos_tbl  : array of Double;      // pCOSvZpe[u
  vsin_tbl  : array of Double;      // pSINvZpe[u
  vcos_tbl  : array of Double;      // pCOSvZpe[u
  buf_x     : array of Double;      // Ɨpobt@ij
  buf_y     : array of Double;      // Ɨpobt@ij
  i         : Integer;
begin
  // z̊m
  SetLength(b_rl, GWidth, GHeight);
  SetLength(b_im, GWidth, GHeight);
  SetLength(hsin_tbl, GWidth);
  SetLength(hcos_tbl, GWidth);
  SetLength(vsin_tbl, GHeight);
  SetLength(vcos_tbl, GHeight);
  SetLength(buf_x, GWidth);
  SetLength(buf_y, GHeight);
  if (b_rl = nil) or (b_im = nil) or (hsin_tbl = nil) or (hcos_tbl = nil) or
     (vsin_tbl = nil) or (vcos_tbl = nil) or (buf_x = nil) or (buf_y = nil) then begin
    Result := -1;
    exit;
  end;

  cstb(GWidth,  inv, hsin_tbl, hcos_tbl);     // pSIN,COSe[u쐬
  cstb(GHeight, inv, vsin_tbl, vcos_tbl);     // pSIN,COSe[u쐬
  // ̂ees
  for i := 0 to GHeight - 1 do
    fft1core(@a_rl[i][0], @a_im[i][0], GWidth, X_EXP, @hsin_tbl[0], @hcos_tbl[0], @buf_x[0]);
  // Qf[^̓]u
  rvmtx1(a_rl, b_rl, GWidth, Gheight);
  rvmtx1(a_im, b_im, GWidth, Gheight);
  // ̂ees
  for i := 0 to GWidth - 1 do
    fft1core(@b_rl[i][0], @b_im[i][0], GHeight, Y_EXP, @vsin_tbl[0], @vcos_tbl[0], @buf_y[0]);
  //* Qf[^̓]u
  rvmtx2(b_rl, a_rl, GWidth, GHeight);
  rvmtx2(b_im, a_im, GWidth, GHeight);
  result := 0;
end;

//--- fftimage --- Qeešʂ摜܂ -----------------------------
//        iGWidthCGHeightQ̗ݏ̏ꍇɌ܂j
//    image_in:    ͉摜z
//    image_out:    o͉摜zieesj
//-----------------------------------------------------------------------------
function TForm1.fftimage(image_in, image_out: B2array): Integer;
var
  ar : D2array;             // f[^io͌p)
  ai : D2array;             // f[^io͌p)
  norm, max : Double;
  data      : Double;
  i, j      : Integer;
  X_EXP, Y_EXP :  Integer;
begin
  setlength(ar, GHeight, GWidth);
  setlength(ai, GHeight, GWidth);
  if (ar = nil) or (ai = nil) then begin
    result := -1;
    exit;
  end;
  // 摜ǂݍ݁CQees̓̓f[^ɕϊ܂
  for i := 0 to GHeight - 1 do
    for j := 0 to GWidth - 1 do begin
      ar[i,  j] := image_in[i][j];
      ai[i, j] := 0.0;
    end;
  // f[^̂Qׂ̂vZ
  X_EXP := 0;
  J := 1;
  for i := 0 to GWidth do begin
    j := J * 2;
    if J > GWidth then begin
      X_EXP := i;
      break;
    end;
  end;
  // f[^̂Qׂ̂vZ
  Y_EXP := 0;
  J := 1;
  for i := 0 to GHeight do begin
    j := J * 2;
    if J > GHeight then begin
      Y_EXP := i;
      break;
    end;
  end;
  // Qeess
  if fft2(ar, ai, 1, Y_EXP, X_EXP) = -1 then begin
    Result := -1;
    exit;
  end;
  // eesʂ摜
  max := 0;
  for i := 0 to GHeight - 1 do begin
    for j := 0 to GWidth - 1 do begin
      norm := ar[i, j] * ar[i, j] + ai[i, j] * ai[i, j];    // UvZ
      if norm <> 0  then norm := ln(norm) / 2.0
                    else norm := 0.0;
      ar[i, j] := norm;
      if norm > max then max := norm;
    end;
  end;
  for i := 0 to GHeight - 1 do
    for j := 0 to  GWidth - 1 do
      ar[i, j] := ar[i, j] * 255 / max;
  // eesʂ摜f[^ɕϊ
  for i := 0 to  GHeight - 1 do
    for j := 0 to GWidth - 1 do begin
      data := ar[i,  j];
      if data > 255 then data := 255;
      if data <   0 then data := 0;
      image_out[i][j] := Round(data);
    end;
  result := 0;
end;

procedure TForm1.FFT2BtnClick(Sender: TObject);
var
  PBA         : PBytearray;
  PFD, WP     : integer;
  X, Y        : Integer;
begin
  FFT2Btn.Enabled := False;
  OPT := 0;                         // OPT = 0@ʏ̂cesi[j
  if (colorradio.ItemIndex = 3) or (colorradio.ItemIndex = 0) then
    fftimage(InpBMat, OutBMat);       // Qeešʂ摜܂
  if (colorradio.ItemIndex = 3) or (colorradio.ItemIndex = 1) then
    fftimage(InpGMat, OutGMat);       // Qeešʂ摜܂
  if (colorradio.ItemIndex = 3) or (colorradio.ItemIndex = 2) then
    fftimage(InpRMat, OutRMat);       // Qeešʂ摜܂
  PFD := 3;
  OutputBitmap.Width  := GWidth;
  OutputBitmap.Height := GHeight;
  for Y := 0 to Gheight - 1 do begin
    PBA := OutputBitmap.ScanLine[Y];
    for X := 0 to GWidth - 1 do begin
      WP := X * PFD;
      PBA[WP]     := 0;
      PBA[WP + 1] := 0;
      PBA[WP + 2] := 0;
      case colorradio.ItemIndex of
         0 :  begin
                PBA[WP]     := OutBMat[y, x];
                PBA[WP + 1] := OutBMat[y, x] div 2;   // ͋PxႢ̂Œǉ
                PBA[WP + 2] := OutBMat[y, x] div 2;   // ͋PxႢ̂Œǉ3
              end;
         1 :  PBA[WP + 1] := OutGMat[y, x];
         2 :  PBA[WP + 2] := OutRMat[y, x];
         3 :  begin
                PBA[WP]     := OutBMat[y, x];
                PBA[WP + 1] := OutGMat[y, x];
                PBA[WP + 2] := OutRMat[y, x];
              end;
      end;
    end;
  end;
  Image1.Canvas.StretchDraw(VRect, OutputBitmap);                // o͘gɕϔ{o
  FFT2Btn.Enabled := True;
end;

//--- fftfilter --- eesɂ摜̃tB^ ----------------------------
//        iGWidthCGHeightQׂ̂̏ꍇɌ܂j
//    image_in:     ͉摜z
//    image_out:    o͉摜z
//    a, b:         ʉߑшiaȏCbȉ̎gʉ߂܂j
//                  a=0Cb>GHeight, GWidthƂCSшʉ
//-----------------------------------------------------------------------------
function TForm1.fftfilter(image_in, image_out: B2array; a, b: Integer): Integer;
var
  ar : D2array;             // f[^io͌p)
  ai : D2array;             // f[^io͌p)
  ff : D2array;             // tB^̋Ԏg
  norm, max   : Double;
  data        : Double;
  i, j, circ  : Integer;
  X_EXP, Y_EXP :  Integer;
begin
  setlength(ar, GHeight, GWidth);
  setlength(ai, GHeight, GWidth);
  setlength(ff, GHeight, GWidth);
  if (ar = nil) or (ai = nil) or (ff = nil) then begin
    result := -1;
    exit;
  end;
  // 摜ǂݍ݁CQees̓̓f[^ɕϊ
  for i := 0 to GHeight - 1 do
    for j := 0 to GWidth - 1 do begin
      ar[i,  j] := image_in[i][j];
      ai[i, j] := 0.0;
    end;
  // f[^̂Qׂ̂vZ
  X_EXP := 0;
  J := 1;
  for i := 0 to GWidth do begin
    j := J * 2;
    if J > GWidth then begin
      X_EXP := i;
      break;
    end;
  end;
  // f[^̂Qׂ̂vZ
  Y_EXP := 0;
  J := 1;
  for i := 0 to GHeight do begin
    j := J * 2;
    if J > GHeight then begin
      Y_EXP := i;
      break;
    end;
  end;
  // Qeess 摜gɕϊ
  if fft2(ar, ai, 1, Y_EXP, X_EXP) = -1 then begin
    Result := -1;
    exit;
  end;
  // gaȏbȉ̐ʉ߂tB^܂
  for i := 0 to GHeight - 1 do
    for j := 0 to GWidth - 1 do begin
      data := (j - GWidth / 2) * (j - GWidth / 2)
                + (i - GHeight / 2) * (i - GHeight / 2);
      circ := trunc(sqrt(data));
      if (circ >= a) and (circ <= b) then
        ff[i, j] := 1.0
      else
        ff[i, j] := 0.0;
    end;

  // 摜̎gɑ΂ătB^sȂ܂
  for i := 0 to GHeight - 1 do
    for j := 0 to GWidth - 1 do begin
      ar[i, j] := ar[i, j] * ff[i, j];
      ai[i, j] := ai[i, j] * ff[i, j];
    end;
  if CheckBox1.Checked then begin
  // eesʂ摜
    max := 0;
    for i := 0 to GHeight - 1 do
      for j := 0 to GWidth - 1 do begin
        norm := ar[i, j] * ar[i, j] + ai[i, j] * ai[i, j];    // UvZ
        if norm <> 0  then norm := ln(norm) / 2.0
                      else norm := 0.0;
        ar[i,  j] := norm;
        if norm > max then max := norm;
      end;
    for i := 0 to GHeight - 1 do
      for j := 0 to  GWidth - 1 do
        ar[i,  j] := ar[i,  j] * 255 / max;
  end
  else begin
    // teessāCtB^ꂽg摜ɖ߂܂
    if fft2(ar, ai, -1, Y_EXP, X_EXP ) = -1 then begin
      Result := -1;
      exit;
    end;
  end;
  // ʂ摜f[^ɕϊ
  for i := 0 to GHeight - 1 do
    for j := 0 to GWidth - 1 do begin
      data := ar[i, j];
      if data > 255 then data := 255;
      if data <   0 then data := 0;
      image_out[i][j] := Round(data);
    end;
  result := 0;
end;

procedure TForm1.FilterBtnClick(Sender: TObject);
var
  PBA         : PBytearray;
  PFD, WP     : integer;
  X, Y        : Integer;
  a, b        : Integer;
begin
  val(LabeledEdit1.Text, a, X);
  if (X <> 0) or (a < 0) then begin
    application.MessageBox('Ⴂ̎gɃ~X܂B','',0);
    exit;
  end;
  val(LabeledEdit2.Text, b, X);
  if (X <> 0) or (b < 0) then begin
    application.MessageBox('̎gɃ~X܂B','',0);
    exit;
  end;
  if (a >= b) then begin
    application.MessageBox('g̑召΂łB','',0);
    exit;
  end;
  FilterBtn.Enabled := False;
  OPT := 1;                             // OPT = 1@wIcesij
  if (colorradio.ItemIndex = 3) or (colorradio.ItemIndex = 0) then
    fftfilter(InpBMat, OutBMat, a, b);
  if (colorradio.ItemIndex = 3) or (colorradio.ItemIndex = 1) then
    fftfilter(InpGMat, OutGMat, a, b);
  if (colorradio.ItemIndex = 3) or (colorradio.ItemIndex = 2) then
    fftfilter(InpRMat, OutRMat, a, b);
  PFD := 3;
  for Y := 0 to Gheight - 1 do begin
    PBA := OutputBitmap.ScanLine[Y];
    for X := 0 to GWidth - 1 do begin
      WP := X * PFD;
      PBA[WP]     := 0;
      PBA[WP + 1] := 0;
      PBA[WP + 2] := 0;
      case colorradio.ItemIndex of
         0 :  begin
                PBA[WP]     := OutBMat[y, x];
                PBA[WP + 1] := OutBMat[y, x] div 2;   // ͋PxႢ̂Œǉ
                PBA[WP + 2] := OutBMat[y, x] div 2;   // ͋PxႢ̂Œǉ
              end;
         1 :  PBA[WP + 1] := OutGMat[y, x];
         2 :  PBA[WP + 2] := OutRMat[y, x];
         3 :  begin
                PBA[WP]     := OutBMat[y, x];
                PBA[WP + 1] := OutGMat[y, x];
                PBA[WP + 2] := OutRMat[y, x];
              end;
      end;
    end;
  end;
  Image1.Canvas.StretchDraw(VRect, OutputBitmap);                // o͘gɕϔ{o
  FilterBtn.Enabled := True;
end;

procedure TForm1.FileOpenBtnClick(Sender: TObject);
var
  WIC         : TWICImage;
  IHeight     : Integer;
  IWidth      : Integer;
  X, Y        : Integer;
  PBA         : PBytearray;
  PFD, WP     : Integer;
  nx2         : Integer;
  XF, YF      : Boolean;
begin
  VRect := Rect(0, 0, Image1.Width, Image1.Height);
  Image1.Canvas.Brush.Style := bsSolid;
  Image1.Canvas.Brush.Color := clBtnface;
  Image1.Canvas.FillRect(VRect);                              // Canvas 摜
  OpenPictureDialog1.Filter := OpenFileFilter;                // t@CI[vtB^[̐ݒ
  if OpenPictureDialog1.Execute then                          // t@Cw肳ꂽ
    begin
      WIC := TWICImage.Create;                                // TWICImage̐
      try
        InFilename := OpenPictureDialog1.FileName;            // t@C̎擾
        WIC.LoadFromFile(InFilename);                         // 摜̓ǂݍ
        GHeight := WIC.Height;                                // 摜擾
        GWidth  := WIC.Width;                                 // 摜
        IWidth  := ImageHW;                                   // o͐C[W1̕
        IHeight := ImageHW;                                   // o͐C[W1̍
        if GHeight <= GWidth then                             // cɂo̓TCYݒ
          IHeight := Round(IWidth * GHeight / GWidth)
        else
          IWidth := Round(IHeight * GWidth / GHeight);
        Image1.Width := IWidth;
        Image1.Height:= IHeight;
        Image1.Picture.Bitmap.SetSize(IWidth, IHeight);
        VRect := Rect(0, 0, IWidth, IHeight);                 // o͘gݒ
//        Image1.Canvas.StretchDraw(VRect, WIC);                // o͘gɕϔ{o
        InputDBitmap.Assign(WIC);                             // Tbitmap  TWICImageRs[
      finally
        WIC.Free;                                             // TWICImage 
      end;
    end
    else exit;
  OutputBitmap.Width  := GWidth;
  OutputBitmap.Height := GHeight;
  setLength(InpBMat, GHeight, GWidth);
  setLength(InpGMat, GHeight, GWidth);
  setLength(InpRMat, GHeight, GWidth);
  setLength(OutBMat, GHeight, GWidth);
  setLength(OutGMat, GHeight, GWidth);
  setLength(OutRMat, GHeight, GWidth);
  PFD := 3;
  if InputDBitmap.PixelFormat = pf32bit then PFD := 4;
  for Y := 0 to Gheight - 1 do begin
    PBA := InputDBitmap.ScanLine[Y];
    for X := 0 to GWidth - 1 do begin
      WP := X * PFD;
      InpBMat[Y, X] := PBA[WP];
      InpGMat[Y, X] := PBA[WP + 1];
      InpRMat[Y, X] := PBA[WP + 2];
    end;
  end;
  for Y := 0 to Gheight - 1 do begin
    PBA := OutputBitmap.ScanLine[Y];
    for X := 0 to GWidth - 1 do begin
      WP := X * 3;
      PBA[WP]     := InpBMat[Y][X];
      PBA[WP + 1] := InpGMat[Y][X];
      PBA[WP + 2] := InpRMat[Y][X];
    end;
    Image1.Canvas.StretchDraw(VRect, OutputBitmap);           // o͘gɕϔ{o
  end;
  XF := False;
  YF := False;
  nx2 := 1;
  repeat
    nx2 := nx2 * 2;
    if nx2 = GWidth then XF := True;
    if nx2 = GHeight then YF := True;
  until (nx2 > GWidth) and (nx2 > GHeight);
  if not XF or not YF  then begin
    if not XF then application.MessageBox('摜̑傫Q̗ݏł͂܂B','',0);
    if not YF then application.MessageBox('摜̑傫Q̗ݏł͂܂B','',0);
    FilterBtn.Enabled := False;
    FFT2Btn.Enabled := False;
    FilterBtn.Enabled := False;
    exit;
  end;
  FilterBtn.Enabled := True;
  FFT2Btn.Enabled := True;
  FilterBtn.Enabled := True;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Image1.Width  := ImageHW;
  Image1.Height := ImageHW;
  InputDBitmap  := TBitmap.Create;
  OutputBitmap  := TBitmap.Create;
  InputDBitmap.PixelFormat := pf24bit;
  OutputBitmap.PixelFormat := pf24bit;
  LabeledEdit1.Text := '0';
  LabeledEdit2.Text := '1000';
  FilterBtn.Enabled := False;
  FFT2Btn.Enabled := False;
  FilterBtn.Enabled := False;
  colorRadio.Items.Add('');
  colorRadio.Items.Add('');
  colorRadio.Items.Add('');
  colorRadio.Items.Add('S');
  colorRadio.ItemIndex := 3;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  InputDBitmap.Free;
  OutputBitmap.Free;
end;

end.
